"""Project files for drf-cached-instances."""

# This will make sure the app is always imported when
# Django starts so that shared_task will use this app.
from .celery import app as celery_app
assert celery_app
