<html>
<body>
<h1>Sample Application for drf_cached_instances</h1>
<p>
  This sample app demonstrates how to integrate
  <a href="https://github.com/jwhitlock/drf_cached_instances">drf_cached_instances</a>
  into a
  <a href="http://www.django-rest-framework.org">Django REST Framework</a>
  app.
</p>
<p>
  Most of the action is in the source code, but you can view the
  <a href="/api/">browsable API</a>.
</p>
</body>
</html>
